@@ -1,11 +1,7 @@
 /*
- * Copyright (c) 2020-2023, Stalwart Labs Ltd.
+ * SPDX-FileCopyrightText: 2020 Stalwart Labs LLC <hello@stalw.art>
  *
- * Licensed under the Apache License, Version 2.0 <LICENSE-APACHE or
- * https://www.apache.org/licenses/LICENSE-2.0> or the MIT license
- * <LICENSE-MIT or https://opensource.org/licenses/MIT>, at your
- * option. This file may not be copied, modified, or distributed
- * except according to those terms.
+ * SPDX-License-Identifier: Apache-2.0 OR MIT
  */
 
 use std::{
@@ -149,14 +145,14 @@ impl<'x> AuthenticationResults<'x> {
     }
 }
 
-impl<'x> Display for AuthenticationResults<'x> {
+impl Display for AuthenticationResults<'_> {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
         f.write_str(self.hostname)?;
         f.write_str(&self.auth_results)
     }
 }
 
-impl<'x> HeaderWriter for AuthenticationResults<'x> {
+impl HeaderWriter for AuthenticationResults<'_> {
     fn write_header(&self, writer: &mut impl Writer) {
         writer.write(b"Authentication-Results: ");
         writer.write(self.hostname.as_bytes());
